#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <pwd.h>
#include <stdint.h>
#include <sys/types.h>
#include <signal.h>
#include <mpegconsts.h>
#include <mpegtimecode.h>
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "tracker.h"
#include "vj-global.h"
#include "vj-comm.h"
#include "vj-effectctrl.h"

int stream_in_running = 0;
int samplelist_reload = 0;


void display(const char format[], ...) {
	va_list args;
	char buf[1024];
	vjcomm *info = vj_comm_get_info();
	va_start(args,format);
	vsnprintf(buf, sizeof(buf) -1, format,args);
	
	sprintf(info->stat_msg, "\n%s", buf);
	gtk_text_insert(GTK_TEXT(info->status_text), NULL, NULL, NULL, info->stat_msg, -1);
}




void
on_MainWindow_destroy                  (GtkObject       *object,
                                        gpointer         user_data)
{
	/* tell goodbye to veejay */
}


void
on_file1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_sample1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_generate_by_module1_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_combo_entry1_activate               (GtkEditable     *editable,
                                        gpointer         user_data) {

					}


void
on_toptoggle_effects_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();
	int active = gtk_toggle_button_get_active(togglebutton);
	sprintf(info->cmd_msg, "%d %d", MSG_CMD_TOGGLE_CHAIN,active);
	vj_comm_write(info->cmd_msg);
	display("Toggle Effect Chain %s", (active==1 ? "on" : "off"));
}


void
on_toptoggle_audio_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();
	sprintf(info->cmd_msg, "%d", MSG_CMD_TOGGLE_AUDIO);
	//write(request_pipe, info->cmd_msg , MESSAGE_SIZE);
	vj_comm_write(info->cmd_msg);
	display("Toggled Audio");
}

void
on_SE_framedup_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();
	GtkWidget *w = lookup_widget(Sample_window , "SE_framedup");
	gint val = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(w));
	sprintf(info->cmd_msg, "%d %d", MSG_CMD_SAMPLE_SET_FRAMEDUP, val);
	vj_comm_write(info->cmd_msg);
	

}
void on_SE_enable_clicked(GtkButton *button, gpointer user_data) {
	vjcomm *info = vj_comm_get_info();
	
	if(info->current_sample_id==0) {
		display("Select a sample from the samplelist first");
	}
	else {
		display("Editing sample %d",info->current_sample_id);
		if(info->selected_sample_type==0) {
		gtk_notebook_set_page(GTK_NOTEBOOK(info->SampleEdit->sample_type), 1);
		}
		else {
			if(info->selected_sample_type==1) {
				gtk_notebook_set_page(GTK_NOTEBOOK(info->SampleEdit->sample_type),2);
			}
		}		

		update_effect_chain_contents();
		update_effect_control_values();
	}


}
void on_PE_enable_clicked(GtkButton *button, gpointer user_data) {
	vjcomm *info = vj_comm_get_info();

	/*
	if(valid_mode_sample()) {
		sprintf(info->cmd_msg, "%d %d", MSG_CMD_TOGGLE_SAMPLEPLAY,0);
		vj_comm_write(info->cmd_msg);
		sprintf(info->stat_msg, "Disabled Sample playback ");
		gtk_text_insert(GTK_TEXT(info->status_text),NULL,NULL,NULL,info->stat_msg,-1);
	}
	sprintf(info->cmd_msg, "%d %d", MSG_CMD_PATTERN_PLAY,1);
	vj_comm_write(info->cmd_msg);
	notebook = lookup_widget(MainWindow, "notebook1");
	gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 3);
	*/
	display("No pattern playback supported yet");
}

void on_SL_disable_samples_clicked(GtkButton *button, gpointer user_data) {
	/*
	sprintf(info->cmd_msg, "%d %d", MSG_CMD_PATTERN_PLAY,0);
	vj_comm_write(info->cmd_msg);

	sprintf(info->cmd_msg, "%d %d", MSG_CMD_TOGGLE_SAMPLEPLAY,0);
	vj_comm_write(info->cmd_msg);
	sprintf(info->stat_msg, "Playing plain video now ");
	gtk_text_insert(GTK_TEXT(info->status_text),NULL,NULL,NULL,info->stat_msg,-1);\
	can be removed
	*/
}	


gboolean
on_timehscale_button_press_event       (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  vjcomm *info = vj_comm_get_info();
  info->hscale_down = 1;
  return FALSE;
}


gboolean
on_timehscale_button_release_event     (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  vjcomm *info = vj_comm_get_info();
  info->hscale_down = 0;
  return FALSE;
}


void
on_Main_skip_to_start_button_clicked   (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	sprintf(info->cmd_msg, "%d", MSG_CMD_SKIPTOSTART);
	vj_comm_write(info->cmd_msg);
}


void
on_Main_fast_reverse_button_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	sprintf(info->cmd_msg, "%d %d", MSG_CMD_FAST_BACKWARD, 15);
	vj_comm_write(info->cmd_msg);
}


void
on_Main_play_reverse_button_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	sprintf(info->cmd_msg, "%d", MSG_CMD_PLAY_BACKWARD);
	vj_comm_write(info->cmd_msg);
}


void
on_Main_frame_backward_button_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	sprintf(info->cmd_msg, "%d %ld", MSG_CMD_SET_FRAME, info->frame_num - 1);
	vj_comm_write(info->cmd_msg);

}


void
on_Main_play_pause_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	sprintf(info->cmd_msg, "%d", MSG_CMD_PLAYPAUSE);
	vj_comm_write(info->cmd_msg);
}


void
on_Main_frame_skip_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	sprintf(info->cmd_msg, "%d %ld", MSG_CMD_SET_FRAME, info->frame_num + 1);
	vj_comm_write(info->cmd_msg);
}


void
on_Main_play_forward_button_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	sprintf(info->cmd_msg, "%d", MSG_CMD_PLAY_FORWARD);
	vj_comm_write(info->cmd_msg);
}


void
on_Main_fast_forward_button_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	sprintf(info->cmd_msg, "%d %d", MSG_CMD_FAST_FORWARD, 15);
	vj_comm_write(info->cmd_msg);
}



void
on_Main_skip_to_end_button_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	sprintf(info->cmd_msg, "%d", MSG_CMD_SKIPTOEND);
	vj_comm_write(info->cmd_msg);
}


void
on_Main_sample_start_button_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	if (!valid_mode_sample() && !valid_mode_tag() &&!valid_mode_pattern()) {
		sprintf(info->cmd_msg, "%d", MSG_CMD_SET_SAMPLESTART);
		vj_comm_write(info->cmd_msg);
		display("Setted sample start at frame %ld",info->frame_num);
		info->new_sample_start = info->frame_num;
	}
	else {
		display("Sample or tag is already playing, use markers.");
	}
}


void
on_Main_sample_end_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	char *text[2];
	vjcomm *info = vj_comm_get_info();

	text[0] = g_new(gchar , 6);

	if(!valid_mode_sample()) {
		sprintf(info->cmd_msg, "%d", MSG_REQ_SET_SAMPLEEND);
		vj_comm_write(info->cmd_msg);
		display("Setted sample end frame at %ld", info->frame_num);
	}
	else {
		display("Sample or tag is already playing. use markers.");
	}
	if(samplelist_reload) {
	gtk_clist_clear(GTK_CLIST(info->SampleList));
	get_all_samples(info->total_samples);
	get_all_tags(info->total_tags);
	samplelist_reload--;
	}
}


void
on_RE_paste_at_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	gint n;
	vjcomm *info = vj_comm_get_info();

	if (info->RawEdit->sel_paste_at>0) {
		sprintf(info->cmd_msg, "%d %d %ld 0", MSG_REQ_EDIT_VIDEO, EDIT_PASTE_AT, 
		info->RawEdit->sel_paste_at);
		vj_comm_write(info->cmd_msg);
		n = vj_comm_read(info->cmd_msg);
		if(n==0) display("Error pasting video: %s",info->cmd_msg);
	}		
	else {
		display("Cannot insert at frame %ld", info->RawEdit->sel_paste_at);
	}
}


gboolean
on_RE_start_frame_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

  info->RawEdit->sel_start_pressed = 1;
  return FALSE;
}


gboolean
on_RE_start_frame_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

 if (info->RawEdit->sel_start_pressed) {
		gint value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
		info->RawEdit->sel_start_at = value;
		display("Setted selection start to frame %ld",value);
		info->RawEdit->sel_start_pressed = 0;		
	}

  return FALSE;
}


gboolean
on_RE_end_frame_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

 info->RawEdit->sel_end_pressed = 1;

  return FALSE;
}


gboolean
on_RE_end_frame_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

 if (info->RawEdit->sel_end_pressed) {
		gint value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
		info->RawEdit->sel_end_at = value;
		display("Setted selection end to frame %d",value);
		info->RawEdit->sel_end_pressed = 0;
	}
  
  return FALSE;
}


gboolean
on_RE_paste_at_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

  info->RawEdit->sel_paste_pressed = 1;
  return FALSE;
}


gboolean
on_RE_paste_at_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

  if (info->RawEdit->sel_paste_pressed) {
		gint value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
		info->RawEdit->sel_paste_at = value;
		display("Paste contents of buffer at %d",value);
		info->RawEdit->sel_paste_pressed = 0;			
	}
  return FALSE;
}


void
on_RE_show_info_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{

}


void
on_RE_cut_button_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	gint n;
	vjcomm *info = vj_comm_get_info();

	if (info->RawEdit->sel_start_at >= 0 && info->RawEdit->sel_end_at > info->RawEdit->sel_start_at) {
		sprintf(info->cmd_msg, "%d %d %ld %ld", MSG_REQ_EDIT_VIDEO, EDIT_CUT, 
			info->RawEdit->sel_start_at, info->RawEdit->sel_end_at);
		//write(request_pipe, info->cmd_msg , MESSAGE_SIZE);
		vj_comm_write(info->cmd_msg);
		//n = read(input_pipe, info->cmd_msg, MESSAGE_SIZE);
		n = vj_comm_read(info->cmd_msg);
	}
	else {
		display("Selection end must be greater than selection start");
	}

}


void
on_RE_copy_button_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
	gint n;
	vjcomm *info = vj_comm_get_info();

	if (info->RawEdit->sel_start_at >= 0 && info->RawEdit->sel_end_at > info->RawEdit->sel_start_at) {
		sprintf(info->cmd_msg, "%d %d %ld %ld", MSG_REQ_EDIT_VIDEO, EDIT_COPY, 
			info->RawEdit->sel_start_at, info->RawEdit->sel_end_at);
		//write(request_pipe, info->cmd_msg , MESSAGE_SIZE);
		vj_comm_write(info->cmd_msg);
		//		n = read(input_pipe, info->cmd_msg, MESSAGE_SIZE);
		n = vj_comm_read(info->cmd_msg);
	}
	else {
		display("Selection end must be greater than selection start");
	}
}


void
on_RE_crop_button_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{	
	gint n;
	vjcomm *info = vj_comm_get_info();

	if (info->RawEdit->sel_start_at >= 0 && info->RawEdit->sel_end_at > info->RawEdit->sel_start_at) {
		sprintf(info->cmd_msg, "%d %d %ld %ld", MSG_REQ_EDIT_VIDEO, EDIT_CROP, 
			info->RawEdit->sel_start_at, info->RawEdit->sel_end_at);
		//write(request_pipe, info->cmd_msg , MESSAGE_SIZE);
		vj_comm_write(info->cmd_msg);
		//n = read(input_pipe, info->cmd_msg, MESSAGE_SIZE);
		n = vj_comm_read(info->cmd_msg);
	}
	else {
		display("Selection end must be greater than selection start");
	}
}


void
on_RE_delete_button_clicked            (GtkButton       *button,
                                        gpointer         user_data)
{
	gint n;
	vjcomm *info = vj_comm_get_info();

	if (info->RawEdit->sel_start_at >= 0 && info->RawEdit->sel_end_at > info->RawEdit->sel_start_at) {
		sprintf(info->cmd_msg, "%d %d %ld %ld", MSG_REQ_EDIT_VIDEO, EDIT_DEL, 
			info->RawEdit->sel_start_at, info->RawEdit->sel_end_at);
		//write(request_pipe, info->cmd_msg , MESSAGE_SIZE);
		vj_comm_write(info->cmd_msg);
		//n = read(input_pipe, info->cmd_msg, MESSAGE_SIZE);
		n = vj_comm_read(info->cmd_msg);
	}
	else {
		display("Selection end must be greater than selection start");
	}

}


void
on_RE_set_sel_start_button_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	info->RawEdit->sel_start_at = info->frame_num;
	display("Setted Selection Start at frame %ld ", info->RawEdit->sel_start_at);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(info->RawEdit->start_frame_spin), info->RawEdit->sel_start_at);
}


void
on_RE_set_sel_end_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	info->RawEdit->sel_end_at = info->frame_num;
	display("Setted selection end at frame %ld", info->RawEdit->sel_end_at);

}	


void projectlist_ok_sel(GtkWidget *widget , GtkFileSelection *fs) {
	gchar *filename;
	gint result=0;
	vjcomm *info = vj_comm_get_info();

	filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	/*
	sprintf(info->cmd_msg, "%d %s", MSG_REQ_PATTERN_LOAD, filename);
	vj_comm_write(info->cmd_msg);
	//n = read(input_pipe, info->cmd_msg, MESSAGE_SIZE);
	n = vj_comm_read(info->cmd_msg);
	sscanf(info->cmd_msg, "%d %d", &req_id, &result);
	*/

}

void
on_SL_open_project_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *filedialog;
	

	filedialog = create_SL_load_project();
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->ok_button),
		"clicked", (GtkSignalFunc) projectlist_ok_sel, filedialog);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->cancel_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filedialog));
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->ok_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filedialog));
	gtk_widget_show(filedialog);

}


void samplelist_ok_sel(GtkWidget *widget, GtkFileSelection *fs) {
	gchar *filename;
	vjcomm *info = vj_comm_get_info();

	gint req_id, result,numsamples;
	filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	sprintf(info->cmd_msg, "%d %s", MSG_REQ_OPEN, filename);
	vj_comm_write(info->cmd_msg);

	//read(input_pipe, info->cmd_msg, MESSAGE_SIZE);
	vj_comm_read(info->cmd_msg);
	sscanf(info->cmd_msg, "%d %d %d", &req_id, &result, &numsamples);
		
	if (result) {
		display("Opened file %s ", filename);
	}
	else {
		display("Error opening file %s ", filename);
	}	
	/* clear list, insert all new samples 
	gtk_clist_freeze(GTK_CLIST(info->SampleList));
	gtk_clist_clear(GTK_CLIST(info->SampleList));

	sprintf(info->cmd_msg, "%d %d %d", MSG_REQ_FLOOD_SAMPLES, 1 , numsamples);
	vj_comm_write(info->cmd_msg);
	for(i=1; i <= numsamples; i++) {
		int row;
		read(input_pipe, info->cmd_msg, MESSAGE_SIZE);
		sscanf(info->cmd_msg, "%s %s %s", id, duration, source);
		text[0] = g_new(gchar,5);
		sprintf(text[0], "%s", id);
		text[1] = NULL;
		text[2] = NULL;

		row = gtk_clist_append(GTK_CLIST(info->SampleList),text);
		gtk_clist_set_text(GTK_CLIST(info->SampleList),row, 1, duration);		
		gtk_clist_set_text(GTK_CLIST(info->SampleList),row, 2, source);	
		
	}
	
	gtk_clist_thaw(GTK_CLIST(info->SampleList));	
	*/
	
}

void
on_SL_open_samplelist_button_clicked   (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *filedialog;
	

	filedialog = create_SL_load_samplelist();
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->ok_button),
		"clicked", (GtkSignalFunc) samplelist_ok_sel, filedialog);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->cancel_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filedialog));
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->ok_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filedialog));
	gtk_widget_show(filedialog);
}


void samplelist_save_sel(GtkWidget *widget, GtkFileSelection *fs) {
	vjcomm *info = vj_comm_get_info();

	gchar *filename;
	
	int req_id, result;
	filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	sprintf(info->cmd_msg, "%d %s", MSG_REQ_SAVE, filename);
	vj_comm_write(info->cmd_msg);
	//	n = read(input_pipe, info->cmd_msg, MESSAGE_SIZE);
	vj_comm_read(info->cmd_msg);
	sscanf(info->cmd_msg, "%d %d", &req_id, &result);
	if(result) display("Wrote EditList %s", filename); 
	else display("Failed to write editlist %s", filename);
}



void on_SL_save_samplelist_clicked( GtkButton *button, gpointer user_data) {
	GtkWidget *filedialog;
	vjcomm *info = vj_comm_get_info();

	filedialog = create_SL_save_samplelist();
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->ok_button),
		"clicked", (GtkSignalFunc) samplelist_save_sel, filedialog);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->cancel_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filedialog));
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->ok_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filedialog));
	gtk_widget_show(filedialog);
	
}

void project_save_sel(GtkWidget *widget, GtkFileSelection *fs) {
	gchar *filename;
	
	int result=0;
	vjcomm *info = vj_comm_get_info();

	filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	/*
	sprintf(info->cmd_msg, "%d %s", MSG_REQ_PATTERN_SAVE, filename);
	vj_comm_write(info->cmd_msg);
	//n = read(input_pipe, info->cmd_msg, MESSAGE_SIZE);
	vj_comm_read(info->cmd_msg);
	sscanf(info->cmd_msg, "%d %d", &req_id, &result);
	*/
	if (result) {
		display("Saved file %s ", filename);
	}
	else {
		display("Saved opening file %s ", filename);
	}	
}

void on_SL_save_project_clicked( GtkButton *button, gpointer user_data) {
	GtkWidget *filedialog;
	

	filedialog = create_SL_save_project();
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->ok_button),
		"clicked", (GtkSignalFunc) project_save_sel, filedialog);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->cancel_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filedialog));
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->ok_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filedialog));
	gtk_widget_show(filedialog);

}

void 
on_SL_remove_video_button_clicked(GtkButton *button, gpointer user_data){

}

void add_video_file_sel(GtkWidget *widget, GtkFileSelection *fs) {
	gchar *filename;
	
	
	vjcomm *info = vj_comm_get_info();

	filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	sprintf(info->cmd_msg, "%d %s", MSG_REQ_ADD_VIDEO_FILE, filename);
	vj_comm_write(info->cmd_msg);
	get_editlist_info();
}

void
on_SL_add_video_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	

	GtkWidget *filedialog;
	

	filedialog = create_SL_add_video_file();
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->ok_button),
		"clicked", (GtkSignalFunc) add_video_file_sel, filedialog);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->cancel_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filedialog));
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->ok_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filedialog));
	gtk_widget_show(filedialog);
	
}


void
on_SL_play_sample_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();
	
	sprintf(info->cmd_msg, "%d %d %d", MSG_CMD_SET_SAMPLE,info->selected_sample,
		info->selected_sample_type);
	display("Play sample %d", info->selected_sample);
	vj_comm_write(info->cmd_msg);

}


void
on_SL_edit_sample_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *notebook;
	vjcomm *info = vj_comm_get_info();

	if(info->selected_sample) {
		sprintf(info->cmd_msg, "%d %d %d", MSG_CMD_SET_SAMPLE,info->selected_sample,
			info->selected_sample_type);
		vj_comm_write(info->cmd_msg);
		display("Edit sample %d",info->selected_sample);
			
		notebook = lookup_widget(Sample_window, "notebook7");
		gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 1);
		gtk_notebook_set_page(GTK_NOTEBOOK(info->SampleEdit->sample_type),
			info->selected_sample_type);
	} 
}


void
on_SL_copy_sample_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar *duration, *filename;
	gchar *text[2];
	gint n,req_id, result;
	gint row;
	vjcomm *info = vj_comm_get_info();

	if (info->selected_sample && info->selected_sample_type==0) {
		sprintf(info->cmd_msg, "%d %d", MSG_REQ_COPY_SAMPLE, info->selected_sample);
		//write(request_pipe, info->cmd_msg,MESSAGE_SIZE);
		vj_comm_write(info->cmd_msg);
		//n = read(input_pipe, info->cmd_msg, MESSAGE_SIZE);
		n = vj_comm_read(info->cmd_msg);
		if(n==0) {
			sscanf(info->cmd_msg, "%d %d", &req_id, &result);
			if (req_id == MSG_REQ_COPY_SAMPLE && result !=-1) {
				text[0] = g_new(gchar, 6);
				sprintf(text[0], "%06d", result);
				text[1] = NULL;
				text[2] = NULL;
				gtk_clist_get_text(GTK_CLIST(info->SampleList), 
					info->selected_row, 1, &duration);
				gtk_clist_get_text(GTK_CLIST(info->SampleList),
					info->selected_row, 2, &filename);
				row = gtk_clist_append(GTK_CLIST(info->SampleList),text);
				gtk_clist_set_text(GTK_CLIST(info->SampleList), row, 1, duration);
				gtk_clist_set_text(GTK_CLIST(info->SampleList), row, 2, filename);
			}
		}
	}
}


void
on_SL_delete_sample_button_clicked     (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	if (info->selected_sample) {
		//if( info->selected_sample_type==0) {
			if (info->selected_sample == info->current_sample_id ) {
				display("Cannot delete current playing sample ");
			}
			else {
				display("Deleting %s %d ",(info->selected_sample_type ? "Tag" : "Sample"), info->selected_sample);
				sprintf(info->cmd_msg, "%d %d %d", MSG_CMD_DEL_SAMPLE, info->selected_sample, info->selected_sample_type);
				vj_comm_write(info->cmd_msg);
				gtk_clist_remove(GTK_CLIST(info->SampleList), info->selected_row);
				info->selected_row = 0;
				info->selected_sample = 0;
				samplelist_reload++;
			}
		/*}
		else {
			display("Not deleting Tag (not supported yet) %d ", info->selected_sample);
		}*/	
	}
}


void
on_SL_delete_all_button_clicked        (GtkButton       *button,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	if(!valid_mode_sample && !valid_mode_pattern) {
	sprintf(info->cmd_msg, "%d", MSG_CMD_DEL_ALL);
	vj_comm_write(info->cmd_msg);
	gtk_clist_freeze(GTK_CLIST(info->SampleList));
	gtk_clist_clear(GTK_CLIST(info->SampleList));
	gtk_clist_thaw(GTK_CLIST(info->SampleList));
	display("Deleted all samples ");

	}
}




void
on_SL_columnedlist_selection_get       (GtkWidget       *widget,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data)
{
	
}


void
on_SL_columnedlist_select_row          (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	gchar *text;	
	gint s_id;
	gchar mode[1];
	vjcomm *info = vj_comm_get_info();

	gtk_clist_get_text(GTK_CLIST(info->SampleList), row, 0, &text);
	sscanf(text, "%1s%04d",mode, &s_id);
	if(g_strncasecmp("T",mode,1)==0) {
		info->selected_sample = s_id;
		info->selected_sample_type = 1;
	}
	else if(g_strncasecmp("S", mode, 1)==0) {
		info->selected_sample = s_id;
		info->selected_sample_type = 0;
		}
	
	info->selected_row = row;
}

gboolean 
on_SE_loop_spin_button_press_event  (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	return FALSE;
}

gboolean 
on_SE_loop_spin_button_release_event  (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample()) {
		sprintf(info->cmd_msg, "%d %d", MSG_CMD_SET_LOOPTYPE,
			gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget)));
		vj_comm_write(info->cmd_msg);
		switch( (gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget)) )) {
			case 0:
				display("Disabled loop ");
				break;
			case 1:
				display("Enabled loop ");
				break;
			case 2:
				display("Enabled PingPong loop ");
				break;
		}
	}
	else {
		msg_no_sample();
	}
	return FALSE;
}

void on_SE_toggle_noloop_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample()) {
		if(gtk_toggle_button_get_active(togglebutton)) {
			sprintf(info->cmd_msg , "%d %d", MSG_CMD_SET_LOOPTYPE, 0);
			vj_comm_write(info->cmd_msg);
		}
	}
	else {
		msg_no_sample(); 
	}	
}
void on_SE_toggle_pingpong_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample()) {
		if(gtk_toggle_button_get_active(togglebutton)) {
			sprintf(info->cmd_msg , "%d %d", MSG_CMD_SET_LOOPTYPE, 2);
			vj_comm_write(info->cmd_msg);
		}
	}
	else {
		msg_no_sample(); 
	}
}
void on_SE_toggle_loop_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample()) {
		if(gtk_toggle_button_get_active(togglebutton)) {
			sprintf(info->cmd_msg , "%d %d", MSG_CMD_SET_LOOPTYPE, 1);
			vj_comm_write(info->cmd_msg);
		}
	}
	else {
		msg_no_sample(); 
	}
}	


gboolean
on_SE_start_frame_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  return FALSE;
}


gboolean
on_SE_start_frame_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
 gint value; 
	vjcomm *info = vj_comm_get_info();

 if(valid_mode_sample()) {
	value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
  if (value >= info->sample_end) {
		value = (info->sample_end-1);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), (gfloat)value);
  }
  sprintf(info->cmd_msg, "%d %d", MSG_CMD_SET_START_FRAME,value);
  //write(request_pipe, info->cmd_msg ,MESSAGE_SIZE);
  vj_comm_write(info->cmd_msg);

  display("Updated sample start to frame %d ", value);
  sprintf(info->cmd_msg, "%d %d %d", MSG_CMD_SET_SAMPLE, info->current_sample_id,0);
  vj_comm_write(info->cmd_msg);
  samplelist_update(); 
  }
  else {
    msg_no_sample();
 }	
  return FALSE;
}


gboolean
on_SE_end_frame_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;
}


gboolean
on_SE_end_frame_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

  if(valid_mode_sample) {
  gint value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
  if (value <= info->sample_start) {
	value = info->sample_start + 1;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), (gfloat)value);
  }
  sprintf(info->cmd_msg, "%d %d", MSG_CMD_SET_END_FRAME,value);
  //write(request_pipe, info->cmd_msg ,MESSAGE_SIZE);
  vj_comm_write(info->cmd_msg);
  display("Updated sample end to frame %d ", value);
  sprintf(info->cmd_msg, "%d %d %d", MSG_CMD_SET_SAMPLE,info->current_sample_id,0);
  //write(request_pipe,info->cmd_msg, MESSAGE_SIZE);
  vj_comm_write(info->cmd_msg);
  samplelist_update();
 }
 else {
	msg_no_sample(); 
 }
  return FALSE;
}


gboolean
on_SE_speed_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

  info->SampleEdit->speed_pressed = 1;
  return FALSE;
}


gboolean
on_SE_speed_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

 if (info->SampleEdit->speed_pressed && valid_mode_sample()) {
  	gint value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
  	sprintf(info->cmd_msg, "%d %d", MSG_CMD_SET_SPEED, value);
  	vj_comm_write(info->cmd_msg);
  	display("Setted playback speed to %d ", value);
	info->SampleEdit->speed_pressed = 0;
 } 
 else {
	msg_no_sample();
 }
 return FALSE;
}


gboolean
on_SE_next_sample_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

  info->SampleEdit->next_sample_pressed = 1;
  return FALSE;
}


gboolean
on_SE_next_sample_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

 
  if(valid_mode_sample()) {
  if (info->SampleEdit->next_sample_pressed) {
	gint value = gtk_spin_button_get_value_as_int(
		GTK_SPIN_BUTTON(widget));
 	sprintf(info->cmd_msg, "%d %d %d", MSG_CMD_SET_SAMPLE_NEXT, 
		info->selected_sample,value);
	vj_comm_write(info->cmd_msg);
	info->SampleEdit->next_sample_pressed = 0;
  }
 } else {
    msg_no_sample();
 }
 
  return FALSE;
}


gboolean
on_SE_max_loops_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  vjcomm *info = vj_comm_get_info();

  info->SampleEdit->max_loops_pressed = 1;
  return FALSE;
}


gboolean
on_SE_max_loops_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

  if(valid_mode_sample()) {
  if (info->SampleEdit->max_loops_pressed) {
	char msg[MESSAGE_SIZE];
	gint value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
	sprintf(msg, "%d %d", MSG_CMD_SET_SAMPLE_LOOPS, value);
	vj_comm_write(msg);	
	display("Setted %d Loops before starting next sample",value);
	info->SampleEdit->max_loops_pressed = 0;
  }
 } else {
	msg_no_sample();
}
  return FALSE;
}


void 
on_SE_description_activate(GtkEditable *editable, gpointer user_data) {
	/* remove me*/
}

void 
on_SE_description_changed(GtkEditable *editable, gpointer user_data) {

	vjcomm *info = vj_comm_get_info();

	GtkWidget *box  = lookup_widget(Sample_window, "SE_description");
	gchar *descr = gtk_entry_get_text(
		GTK_ENTRY(box)
	);
	if(descr==NULL || strlen(descr)<=0) sprintf(descr," ");
	sprintf(info->cmd_msg, "%d %s",MSG_CMD_SET_SAMPLEDESCR,
		descr);
	vj_comm_write(info->cmd_msg);
	samplelist_update();
}

void
on_SE_ImageEffects_add_button_clicked  (GtkButton       *button,
                                        gpointer         user_data) {

	gchar *effect_descr;
	gint effect_id, req_id, chain_id, params;
	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample() || valid_mode_tag()) {
		//effect_descr = gtk_entry_get_text(
		//		GTK_ENTRY(GTK_COMBO(
		//			info->SampleEdit->combo_box_image_effects)
		//		->entry)
		//);
		if(info->SampleEdit->new_image_effect>=0) {
		gtk_clist_get_text(GTK_CLIST(info->SampleEdit->image_effects),
			info->SampleEdit->new_image_effect,0,&effect_descr);
		
		effect_id = vj_effect_get_effect_nr(effect_descr);
		sprintf(info->cmd_msg, "%d %d", MSG_REQ_ADD_EFFECT,effect_id);
		//write(request_pipe,info->cmd_msg,MESSAGE_SIZE);
		vj_comm_write(info->cmd_msg);
		//read(input_pipe,info->cmd_msg,MESSAGE_SIZE);
		vj_comm_read(info->cmd_msg);
		sscanf(info->cmd_msg, "%d %d %d", &req_id, &chain_id, &params);
		if(req_id==MSG_REQ_ADD_EFFECT) {
			display("Added Image Effect '%s' on row %d ", effect_descr, chain_id);
			gtk_clist_set_text(GTK_CLIST(info->SampleEdit->effect_chain),chain_id,1,effect_descr);
			info->SampleEdit->selected_effect_id = effect_id;
			update_effect_control_values();
		}
		else {
			display("Unexpected replay: %d ",req_id);
		}
		}
	}
	else {
		msg_no_sample();
	}
}

void
on_SE_VideoEffects_add_button_clicked  (GtkButton       *button,
                                        gpointer         user_data) {
	gchar *effect_descr;
	gint effect_id, req_id, chain_id , params;
	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample() || valid_mode_tag()) {
		if(info->SampleEdit->new_video_effect >= 0) {
		//effect_descr = gtk_entry_get_text(
		//		GTK_ENTRY(GTK_COMBO(
		//			info->SampleEdit->combo_box_video_effects)
		//		->entry)
		//);
		gtk_clist_get_text(GTK_CLIST(info->SampleEdit->video_effects),
			info->SampleEdit->new_video_effect,0,&effect_descr);
		

		effect_id = vj_effect_get_effect_nr(effect_descr);
		sprintf(info->cmd_msg, "%d %d", MSG_REQ_ADD_EFFECT,effect_id);
		//write(request_pipe,info->cmd_msg,MESSAGE_SIZE);
		vj_comm_write(info->cmd_msg);
		//read(input_pipe,info->cmd_msg,MESSAGE_SIZE);
		vj_comm_read(info->cmd_msg);
		sscanf(info->cmd_msg, "%d %d %d", &req_id, &chain_id, &params);
	
		if(req_id==MSG_REQ_ADD_EFFECT) {
			display("Added Video Effect '%s' on row %d ", effect_descr,chain_id);
			gtk_clist_set_text(GTK_CLIST(info->SampleEdit->effect_chain),chain_id,1,effect_descr);
			info->SampleEdit->selected_effect_id = effect_id;
			update_effect_control_values();
		}
		else {
			display("Unexpected replay: %d ",req_id);
		}
		}
	}
	else {
		msg_no_sample();
	}
		
}


void
on_SE_EffectControl_P0_spinbutton_changed
                                        (GtkEditable     *editable,
                                        gpointer         user_data) {

}


gboolean
on_SE_EffectControl_P0_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data){
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P0_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	update_effect_control(widget,0);
	info->SampleEdit->slider_down = 0;
	return FALSE;
}


void
on_SE_EffectControl_P1_spinbutton_changed
                                        (GtkEditable     *editable,
                                        gpointer         user_data) {
}

gboolean
on_SE_EffectControl_P1_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P1_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	update_effect_control(widget,1);
	info->SampleEdit->slider_down = 0;
	return FALSE;
}


void
on_SE_EffectControls_P2_spinbutton_changed
                                        (GtkEditable     *editable,
                                        gpointer         user_data) {
}

gboolean
on_SE_EffectControls_P2_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data){
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControls_P2_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
						
	vjcomm *info = vj_comm_get_info();

	update_effect_control(widget, 2);
	info->SampleEdit->slider_down = 0;
	return FALSE;
}


void
on_SE_EffectControl_P3_spinbutton_changed
                                        (GtkEditable     *editable,
                                        gpointer         user_data) {

}

gboolean
on_SE_EffectControl_P3_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P3_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	update_effect_control(widget, 3);
	info->SampleEdit->slider_down = 0;
	return FALSE;
}

void
on_SE_EffectControl_P4_spinbutton_changed
                                        (GtkEditable     *editable,
                                        gpointer         user_data) {

}

gboolean
on_SE_EffectControl_P4_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P4_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	update_effect_control(widget, 4);
	info->SampleEdit->slider_down = 0;
	return FALSE;
}

void
on_SE_EffectControl_P5_spinbutton_changed
                                        (GtkEditable     *editable,
                                        gpointer         user_data) {


}

gboolean
on_SE_EffectControl_P5_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                     	                   gpointer         user_data){
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P5_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data){
	vjcomm *info = vj_comm_get_info();

	update_effect_control(widget, 5);
	info->SampleEdit->slider_down = 0;
	return FALSE;
}

void
on_SE_EffectControl_P7_spinbutton_changed
                                        (GtkEditable     *editable,
                                        gpointer         user_data){
	

}

gboolean
on_SE_EffectControl_P7_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data){
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P7_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {

	vjcomm *info = vj_comm_get_info();
	update_effect_control(widget, 7);
	info->SampleEdit->slider_down = 0;
	return FALSE;
}

void
on_SE_EffectControl_P6_spinbutton_changed
                                        (GtkEditable     *editable,
                                        gpointer         user_data) {

}

gboolean
on_SE_EffectControl_P6_spinbutton_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data){
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;
}


gboolean
on_SE_EffectControl_P6_spinbutton_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data){
	vjcomm *info = vj_comm_get_info();

	update_effect_control(widget, 6);
	info->SampleEdit->slider_down = 0;
	return FALSE;
}

void
on_SE_clear_entry_button_clicked       (GtkButton       *button,
                                        gpointer         user_data){

	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample() || valid_mode_tag()) {
		sprintf(info->cmd_msg, "%d", MSG_CMD_DEL_CHAINENTRY);
		//write(request_pipe,info->cmd_msg, MESSAGE_SIZE);
		vj_comm_write(info->cmd_msg);
		gtk_clist_set_text(GTK_CLIST(info->SampleEdit->effect_chain),info->chain_entry,1,"<none>");
	}
	else {
		msg_no_sample();
	}
 
}

void
on_SE_EffectControl_dup_entry_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data){

	int result, req_id;
	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample() && info->SampleEdit->selected_effect_id !=-1) {
		sprintf(info->cmd_msg, "%d", MSG_REQ_DUP_CHAIN_ENTRY);
		//write(request_pipe,info->cmd_msg, MESSAGE_SIZE);
		vj_comm_write(info->cmd_msg);
		//read(input_pipe, info->cmd_msg, MESSAGE_SIZE);
		vj_comm_read(info->cmd_msg);
		sscanf(info->cmd_msg, "%d %d", &req_id, &result);
		if(req_id==MSG_REQ_DUP_CHAIN_ENTRY) {
			if(result<0) {
				display("Failure duplicating entry %d ",info->chain_entry);
			}
			else {
				gtk_clist_set_text(GTK_CLIST(info->SampleEdit->effect_chain),
					result,1,
					vj_effect_get_effect_descr(info->SampleEdit->selected_effect_id));

				display("Duplicated entry %d to entry %d ",info->chain_entry,result);
			}	
		}
	}
	else {
		msg_no_sample();
	}

}


void
on_SE_reset_button_clicked             (GtkButton       *button,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample() || valid_mode_tag()) {
		int i;
		sprintf(info->cmd_msg, "%d", MSG_CMD_RESET_CHAIN_ENTRY);
		vj_comm_write(info->cmd_msg);
		for(i=0; i < SAMPLE_MAX_EFFECTS; i++) {
			gtk_clist_set_text(GTK_CLIST(info->SampleEdit->effect_chain),i,1,"<none>");
			info->SampleEdit->selected_effect_id = -1;
		}
		update_effect_control_values();
		clear_parameters();

	}

}
void
on_SE_set_defaults_button_clicked      (GtkButton       *button,
                                        gpointer         user_data) {

	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample() || valid_mode_tag()) {
		sprintf(info->cmd_msg, "%d", MSG_CMD_SET_ENTRY_DEFAULTS);
		vj_comm_write(info->cmd_msg);
	}
	

}


GtkWidget*
tracker_new (gchar *widget_name, gchar *string1, gchar *string2,
                gint int1, gint int2)
{
	return trackerp_new();
}


gboolean
on_pattern_hscroll_button_press_event  (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;
}


gboolean
on_pattern_hscroll_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;
}


gboolean
on_pattern_vscroll_button_press_event  (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;
}


gboolean
on_pattern_vscroll_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;
}

void
on_SE_effect_chain_row_move            (GtkCList        *clist,
                                        gint             arg1,
                                        gint             arg2,
                                        gpointer         user_data){ 

}

void
on_SE_effect_chain_select_all          (GtkCList        *clist,
                                        gpointer         user_data) {


}

void
on_SE_effect_chain_select_row          (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data) {


}


void
on_SE_chain_clist_select_row           (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data) {

	
	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample() || valid_mode_tag()) {
		sprintf(info->cmd_msg, "%d %d", MSG_REQ_SET_CHAINENTRY, row);
		vj_comm_write(info->cmd_msg);
	}
}	

void on_quit1_activate(GtkMenuItem *menuitem, gpointer user_data) {
	vj_comm_exit();
}

void on_SE_playlist_toggle_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
	gint req_id=0, result;
	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample()) {

		sprintf(info->cmd_msg, "%d", MSG_REQ_TOGGLE_PLAYLIST);
		vj_comm_write(info->cmd_msg);
	}

}



void
on_SE_toggle_audio_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{

}


gboolean
on_SE_sample_volume_spin_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;
}


gboolean
on_SE_sample_volume_spin_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{



	if(valid_mode_sample()) {
	vjcomm *info = vj_comm_get_info();

		gint value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
		sprintf(info->cmd_msg, "%d %d", MSG_CMD_SAMPLE_SET_VOLUME,value);
		vj_comm_write(info->cmd_msg);
	}

  return FALSE;
}


gboolean
on_SE_Sample_Volume_Hscale_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;
}


gboolean
on_SE_Sample_Volume_Hscale_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{

  return FALSE;
}



void
on_togglebutton1_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample() || valid_mode_tag()) {
		if(info->SampleEdit->bypass_update==0) {
			if(gtk_toggle_button_get_active(togglebutton)) {
				sprintf(info->cmd_msg, "%d %d", MSG_CMD_BYPASS_CHAIN_ENTRY,1);
			}
			else {
				sprintf(info->cmd_msg, "%d %d", MSG_CMD_BYPASS_CHAIN_ENTRY,0);
			}
			vj_comm_write(info->cmd_msg);
		}
	}
}

void editlist_save_sel(GtkWidget *widget, GtkFileSelection *fs) {
	gchar *filename;
	
	int req_id, result;
	vjcomm *info = vj_comm_get_info();

	filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs));
	sprintf(info->cmd_msg, "%d %s", MSG_REQ_SAVE_EDITLIST, filename);
	vj_comm_write(info->cmd_msg);
	//n = read(input_pipe, info->cmd_msg, MESSAGE_SIZE);
	vj_comm_read(info->cmd_msg);
	sscanf(info->cmd_msg, "%d %d", &req_id, &result);
	if(result) display("Saved EditList %s",filename);
	else display("Failed to write EditList %s",filename);
}


void
on_SL_save_editlist_clicked            (GtkButton       *button,
                                        gpointer         user_data) {

	GtkWidget *filedialog;

	filedialog = create_SL_save_editlist();
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->ok_button),
		"clicked", (GtkSignalFunc) editlist_save_sel, filedialog);
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->cancel_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filedialog));
	gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(filedialog)->ok_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(filedialog));
	gtk_widget_show(filedialog);
	
	

}

void on_editclist_select_row(GtkCList *clist, gint row, gint column, GdkEvent *event,gpointer user_data) {

}

void on_editclist_selection_get(GtkWidget *widget, GtkSelectionData *data, guint info, guint time, gpointer
	user_data) {

}


gboolean
on_SE_EffectControl_P4_Vscale_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;

}

gboolean
on_SE_EffectControl_P4_Vscale_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {

	vjcomm *info = vj_comm_get_info();

	gint val = (gint)(GTK_ADJUSTMENT(GTK_RANGE(widget)->adjustment)->value *
		 vj_effect_get_max_value(info->current_effect_id,4)/100);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(info->SampleEdit->spinbox_p4), val);

	info->SampleEdit->slider_down = 0;

	return FALSE;
}

gboolean
on_SE_EffectControl_P5_Vscale_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {

	vjcomm *info = vj_comm_get_info();
	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P5_Vscale_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	gint val = (gint)(GTK_ADJUSTMENT(GTK_RANGE(widget)->adjustment)->value *
		 vj_effect_get_max_value(info->current_effect_id,5)/100);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(info->SampleEdit->spinbox_p5), 
		val);

	info->SampleEdit->slider_down = 0;

	return FALSE;
}

gboolean
on_SE_EffectControl_P6_Vscale_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P6_Vscale_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	gint val = (gint)(GTK_ADJUSTMENT(GTK_RANGE(widget)->adjustment)->value *
		 vj_effect_get_max_value(info->current_effect_id,6)/100);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(info->SampleEdit->spinbox_p6), 
		val);

	info->SampleEdit->slider_down = 0;

	return FALSE;
}

gboolean
on_SE_EffectControl_P7_Vscale_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P7_Vscale_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	gint val = (gint)(GTK_ADJUSTMENT(GTK_RANGE(widget)->adjustment)->value *
		 vj_effect_get_max_value(info->current_effect_id,7)/100);

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(info->SampleEdit->spinbox_p7), 
		val);
	info->SampleEdit->slider_down = 0;

	return FALSE;
}



gboolean
on_SE_EffectControl_P1_Vscale_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {

	vjcomm *info = vj_comm_get_info();
	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P1_Vscale_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	gint val = (gint)(GTK_ADJUSTMENT(GTK_RANGE(widget)->adjustment)->value *
		 vj_effect_get_max_value(info->current_effect_id,1)/100);

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(info->SampleEdit->spinbox_p1), 
		val);
	info->SampleEdit->slider_down = 0;


	return FALSE;
}

gboolean
on_SE_EffectControl_P2_Vscale_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P2_Vscale_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	gint val = (gint)(GTK_ADJUSTMENT(GTK_RANGE(widget)->adjustment)->value *
		 vj_effect_get_max_value(info->current_effect_id,2)/100);

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(info->SampleEdit->spinbox_p2),
		val);
	info->SampleEdit->slider_down = 0;

	return FALSE;	
}

gboolean
on_SE_EffectControl_P3_Vscale_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {

	vjcomm *info = vj_comm_get_info();
	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P3_Vscale_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	gint val = (gint)(GTK_ADJUSTMENT(GTK_RANGE(widget)->adjustment)->value *
		 vj_effect_get_max_value(info->current_effect_id,3)/100);

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(info->SampleEdit->spinbox_p3), 
		val);
	info->SampleEdit->slider_down = 0;

	return FALSE;
}

gboolean
on_SE_EffectControl_P0_Vscale_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();


	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P0_Vscale_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();


	gint val = (gint)(GTK_ADJUSTMENT(GTK_RANGE(widget)->adjustment)->value *
		 vj_effect_get_max_value(info->current_effect_id,0)/100);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(info->SampleEdit->spinbox_p0), 
		val);
	info->SampleEdit->slider_down = 0;

	return FALSE;
}


void
on_SE_enable_subchain_toggled          (GtkToggleButton *togglebutton,  
					gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();


        if (gtk_toggle_button_get_active(togglebutton)) {
		sprintf(info->cmd_msg, "%d %d", MSG_REQ_SET_SUBCHAIN,1);
		//write(request_pipe,info->cmd_msg,MESSAGE_SIZE);
		vj_comm_write(info->cmd_msg);
	}
	else {
		sprintf(info->cmd_msg, "%d %d", MSG_REQ_SET_SUBCHAIN,0);
		vj_comm_write(info->cmd_msg);
	}
                           

}
gboolean
on_SE_EffectControl_P8_Vscale_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P8_Vscale_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	gint val = (gint)(GTK_ADJUSTMENT(GTK_RANGE(widget)->adjustment)->value *
		 vj_effect_get_max_value(info->current_effect_id,8)/100);

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(info->SampleEdit->spinbox_p8), 
		val);

	info->SampleEdit->slider_down = 0;

	return FALSE;
}
gboolean
on_SE_EffectControl_P8_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data){
	vjcomm *info = vj_comm_get_info();

	info->SampleEdit->slider_down = 1;
	return FALSE;
}

gboolean
on_SE_EffectControl_P8_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	update_effect_control(widget,8);
	info->SampleEdit->slider_down = 0;
	return FALSE;
}


void
on_SE_EffectControl_P8_changed
                                        (GtkEditable     *editable,
                                        gpointer         user_data) {

}


void
on_SE_ImageEffect_List_select_row      (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{

	vjcomm *info = vj_comm_get_info();
	info->SampleEdit->new_image_effect = row;
	
}


void
on_SE_ImageEffect_List_selection_get   (GtkWidget       *widget,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data)
{

}


void
on_SE_VideoEffects_List_select_row     (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{

	vjcomm *info = vj_comm_get_info();
	info->SampleEdit->new_video_effect = row;
}


void
on_SE_VideoEffects_List_selection_get  (GtkWidget       *widget,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data)
{

}

void
on_SE_source_selection_selection_get   (GtkWidget       *widget,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data) {

}

void
on_SE_channel_spin_changed             (GtkEditable     *editable,
                                        gpointer         user_data) {

					}

gboolean
on_SE_channel_spin_button_press_event  (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {

	return FALSE;
					}

gboolean
on_SE_channel_spin_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	if(valid_mode_sample() || valid_mode_tag()) {				
		gint value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
		sprintf(info->cmd_msg, "%d %d", MSG_CMD_SET_SAMPLE_CHANNEL, value);
		vj_comm_write(info->cmd_msg);
		info->SampleEdit->channel_number = value;
	}
	return FALSE;
}


void
on_top_setplaymode_button_clicked      (GtkButton       *button,
                                        gpointer         user_data){
	gchar *mode;
	char msg[MESSAGE_SIZE];
	vjcomm *info = vj_comm_get_info();

	mode = gtk_entry_get_text( GTK_ENTRY(GTK_COMBO(info->playmode_box)->entry) );


	if(g_strncasecmp(mode, "Plain Video",4)==0) {
		sprintf(msg, "%d %d", MSG_CMD_SET_PLAYMODE, VJ_PLAYBACK_MODE_PLAIN);
		//write(request_pipe, msg, MESSAGE_SIZE);
		vj_comm_write(msg);		
	}
	if(g_strncasecmp(mode, "Sample",4)==0) {
		sprintf(msg, "%d %d", MSG_CMD_SET_PLAYMODE, VJ_PLAYBACK_MODE_SAMPLE);
		//write(request_pipe, msg, MESSAGE_SIZE);
		vj_comm_write(msg);
	}
	if(g_strncasecmp(mode, "Pattern",4)==0) {
		sprintf(msg, "%d %d", MSG_CMD_SET_PLAYMODE, VJ_PLAYBACK_MODE_PATTERN);
		//write(request_pipe, msg, MESSAGE_SIZE);
		vj_comm_write(msg);
	}
	if(g_strncasecmp(mode, "Tag",3)==0) {
		sprintf(msg, "%d %d", MSG_CMD_SET_PLAYMODE, VJ_PLAYBACK_MODE_TAG);
		//write(request_pipe, msg, MESSAGE_SIZE);
		vj_comm_write(msg);
	}


}

void
on_SE_set_source_button_clicked        (GtkButton       *button,
                                        gpointer         user_data){

	gchar *mode;
	char msg[MESSAGE_SIZE];
	vjcomm *info = vj_comm_get_info();

	mode = gtk_entry_get_text( GTK_ENTRY(GTK_COMBO(info->SampleEdit->source)->entry) );


	if(g_strncasecmp(mode, "SAMPLE",4)==0) {
		sprintf(msg, "%d %d", MSG_CMD_SET_SAMPLE_SOURCE, 0);
		//write(request_pipe, msg, MESSAGE_SIZE);
		vj_comm_write(msg);
		info->SampleEdit->source_number = 0;
	}
	if(g_strncasecmp(mode, "TAG",4)==0) {
		sprintf(msg, "%d %d", MSG_CMD_SET_SAMPLE_SOURCE, 1);
		//write(request_pipe, msg, MESSAGE_SIZE);
		vj_comm_write(msg);
		info->SampleEdit->source_number = 1;
	}
}

void
on_topcombo_set_playmode_add           (GtkWidget       *widget,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data) {
}

void
on_top_videodump_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();

	sprintf(info->cmd_msg, "%d gyuv-dump", MSG_CMD_TOGGLE_OUTSTREAM);
	vj_comm_write(info->cmd_msg);	
}

void on_TAG_activate_toggled		(GtkToggleButton *togglebutton,
					gpointer user_data) {
	vjcomm *info = vj_comm_get_info();
	if(togglebutton->active) {
		sprintf(info->cmd_msg, "%d %d",
			MSG_CMD_SET_TAG_ACTIVE,1);
	}
	else {
		sprintf(info->cmd_msg,"%d %d", MSG_CMD_SET_TAG_ACTIVE,0);	
	}
	//write( request_pipe, info->cmd_msg,MESSAGE_SIZE);	
	vj_comm_write(info->cmd_msg);

}
gboolean
on_window1_destroy_event               (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data) {

	return FALSE;
}

void
on_MB_select_export_format_selection_get
                                        (GtkWidget       *widget,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data) {
}
					

void
on_combo_entry3_activate               (GtkEditable     *editable,
                                        gpointer         user_data) {
	/*
	GtkWidget *textinfo = lookup_widget(SettingsWindow, "MB_infobox");
	GtkWidget *combo = lookup_widget(SettingsWindow, "MB_select_export_format");
	sprintf(msg, "Exporting Video in [%s] format ",
		gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo)->entry)));
	gtk_text_insert(GTK_TEXT(textinfo), NULL,NULL,NULL, msg, -1);*/

	
}

void
on_MB_set_export_folder_activate       (GtkEditable     *editable,
                                        gpointer         user_data) {
	/*
	gchar *folder_name = gtk_entry_get_text(GTK_ENTRY(info->settings->folder));
	char cwd[1024];
	GtkWidget *textinfo = lookup_widget(SettingsWindow, "MB_infobox");
	struct passwd *pwd;
	char tmpdir[1024];
	pid_t pid;
	int uid;

	if(getwd(cwd)==NULL) {
		sprintf(info->stat_msg , "Error getting path ");
		gtk_text_insert(GTK_TEXT(textinfo),NULL,NULL,NULL,info->stat_msg,-1);
		}

	if( chdir(folder_name)==0) {
		sprintf(info->stat_msg, "Folder %s is a valid folder ",
			folder_name);
		chdir(cwd);
	}
	else {
		uid = getuid();
		pwd = getpwuid(uid); 
		sprintf(tmpdir, "%s/.veejay/tmp", pwd->pw_dir);
	
		gtk_entry_set_text(GTK_ENTRY(info->settings->folder),
			tmpdir);	
		sprintf(info->stat_msg, "Folder is not a valid folder, using defaults ");
		gtk_text_insert(GTK_TEXT(textinfo),NULL,NULL,NULL,info->stat_msg,-1);
	}

	sprintf(info->stat_msg, "Exporting Video to folder %s ",folder_name);
	gtk_text_insert(GTK_TEXT(textinfo),NULL,NULL,NULL,info->stat_msg,-1);	
	*/
}

void
on_MB_check_auto_load_toggled          (GtkToggleButton *togglebutton,
                                        gpointer         user_data) {

	/*
	GtkWidget *textinfo = lookup_widget(SettingsWindow, "MB_infobox");

	if (GTK_TOGGLE_BUTTON(togglebutton)->active) {
				sprintf(info->stat_msg, "When the video is rendered, it will be appended to the Editlist ");	
			}
	else {
		sprintf(info->stat_msg, "Rendered Samples will not be appended to the EditList ");
	}
	gtk_text_insert(GTK_TEXT(textinfo), NULL,NULL,NULL, info->stat_msg, -1);
	*/
}


gboolean
on_MB_set_mjpeg_quality_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {

	

	return FALSE;
					}

gboolean
on_MB_set_mjpeg_quality_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {

	return FALSE;
					}

void
on_MB_set_filename_activate            (GtkEditable     *editable,
                                        gpointer         user_data) {

			}


void
on_Tag_Create_v4l_clicked              (GtkButton       *button,
                                        gpointer         user_data) { }	



void
on_MB_set_export_folder_changed        (GtkEditable     *editable,
                                        gpointer         user_data) {

}
	

void
on_MB_set_filename_changed             (GtkEditable     *editable,
                                        gpointer         user_data) {

}

void
on_export_sample1_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {
	



}

void
on_setup1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {

	gtk_widget_show(TagWindow);

}


gboolean
on_SE_EffectControls_chain_volume_slider_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	return FALSE;

}

gboolean
on_SE_EffectControls_chain_volume_slider_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	return FALSE;

	
}

gboolean
on_SE_EffectControl_chain_volume_spin_button_press_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {

	return FALSE;
}

gboolean
on_SE_EffectControl_chain_volume_spin_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {

	if(valid_mode_sample()) {
	vjcomm *info = vj_comm_get_info();

		gint value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
		sprintf(info->cmd_msg, "%d %d", MSG_CMD_SET_CHAIN_AUDIO_VOLUME,value);
		vj_comm_write(info->cmd_msg);
	}
	return FALSE;
}


void
on_SE_chain_toggle_audio_toggled       (GtkToggleButton *togglebutton,
                                        gpointer         user_data) {

	if(valid_mode_sample()) {
	vjcomm *info = vj_comm_get_info();

		if(gtk_toggle_button_get_active(togglebutton)) {
			sprintf(info->cmd_msg , "%d %d", MSG_CMD_SET_CHAIN_AUDIO, 1);
			vj_comm_write(info->cmd_msg);
		}
		else {
			sprintf(info->cmd_msg , "%d %d", MSG_CMD_SET_CHAIN_AUDIO, 0);
			vj_comm_write(info->cmd_msg);


		}
	}
	else {
		msg_no_sample(); 
	}	

}


void
on_SE_render_to_entry_changed          (GtkEditable     *editable,
                                        gpointer         user_data) {

}

void
on_SE_render_list_select_row           (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data) {

	char msg[MESSAGE_SIZE];
	gchar *text;
	vjcomm *info = vj_comm_get_info();		
	sprintf(msg, "%d %d", MSG_CMD_SET_SAMPLE_RENDER_ENTRY,row);

	gtk_clist_get_text(GTK_CLIST(info->render->list), row, 1, &text);

	if(g_strncasecmp("Empty",text,4)==0) {
		if(row==0) { /* do not override original sample */
			sprintf(msg, "%d %d", MSG_CMD_SET_SAMPLE_RENDER_ENTRY,0);
			vj_comm_write(msg);
		}

	}
	else {
	//	if(g_strncasecmp("Rendering",text,8)==0) {
	//		display("Cannot play while rendering");
	//	}
	//	else {
			vj_comm_write(msg);
	//	}
	}
}

void
on_SE_render_entry_activate_clicked    (GtkButton       *button,
                                        gpointer         user_data) {}


void
on_SE_RenderDelete_clicked             (GtkButton       *button,
                                        gpointer         user_data) { 
	char msg[MESSAGE_SIZE];
	vjcomm *info = vj_comm_get_info();
	display("Deleting entry %d of sample %d",info->active_render_entry, info->current_sample_id);
	sprintf(msg,"%d %d %d", MSG_CMD_SAMPLE_RENDER_DEL, info->current_sample_id, info->active_render_entry);
	vj_comm_write(msg);
}

void
on_SE_RenderNewSample_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{	
	char msg[MESSAGE_SIZE];
	vjcomm *info = vj_comm_get_info();
	display("Moving entry %d of sample %d",info->active_render_entry, info->current_sample_id);

	sprintf(msg, "%d %d %d", MSG_CMD_SAMPLE_RENDER_MOVE, info->current_sample_id, info->active_render_entry);
	vj_comm_write(msg);	
}

void
on_SE_render_button_clicked            (GtkButton       *button,
                                        gpointer         user_data) {

	
	char render_to[5];
	char filename[255];
	char editlist[255];
	char samplelist[255];
	char msg[MESSAGE_SIZE];
	char cmd[MESSAGE_SIZE];
	char port_id[5];
	char host[255];
	char sample_id[10];
	char slist[300];
	char output[300];
        char template[100];
	int ok=-1,res=-1;
	vjcomm *info = vj_comm_get_info();
	int entry = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(info->render->spin));


	if(!valid_mode_sample()) {
		display("You cannot render from tags yet.");
		 return;
	}
	if(entry == info->active_render_entry) {
		display("You are playing entry %d , choose another entry", entry);
		return;
	}

	/*
	sprintf(cmd, "%d %d", MSG_CMD_RENDER_SAMPLE_TO, entry);
	vj_comm_write(cmd);
	*/
	
	sprintf(render_to, "-r %1d",entry);
	sprintf(template, "sample-%d-r%d-XXXXXX",info->current_sample_id,entry);
	if(mkstemp(template)==-1) {
		display("Cannot create a temporary filename for this sample");
		return;
	}
	
	sprintf(filename , "%s/%s.avi", info->veejay_path,template);
	sprintf(editlist, "%s/editlist-%d-r%d.avi", info->veejay_path,info->current_sample_id,entry);
	sprintf(samplelist,"%s/samplelist-%d-r%d.sl",info->veejay_path, info->current_sample_id,entry);
	

	sprintf(msg, "%d %d", MSG_REQ_GET_RENDERING_ENTRY, entry);
	vj_comm_write(msg);
	vj_comm_read(msg);
	sscanf(msg,"%d %d", &res, &ok);
	if(ok != 0) {
		display("veejay is already rendering entry %d of of current smple",entry);
		 return;
	}

	/* now claim this entry. (safety lock) */
	sprintf(msg, "%d %d %d %d", MSG_CMD_SAMPLE_SET_RENDERING_ENTRY, entry,1,info->current_sample_id); 
	vj_comm_write(msg);

	gtk_clist_set_text(GTK_CLIST(info->render->list), entry, 1, "Empty");
        

	/* we could do some more checking, but we wouldn't be here ,
	   so start saving samplelist,editlist*/
	sprintf(msg, "%d %s", MSG_REQ_SAVE, samplelist);
	vj_comm_write(msg);
	vj_comm_read(msg);
	sscanf(msg, "%d %d", &res,&ok);
	if(ok!=1) {
		display("Unable to save the samplelist to %s",samplelist);
		return;	
	}
	sprintf(msg, "%d %s", MSG_REQ_SAVE_EDITLIST,editlist);
	vj_comm_write(msg);
	vj_comm_read(msg);
	sscanf(msg, "%d %d", &res,&ok);
	if(ok!=1) {
		display("Unable to save the editlist to %s",editlist);
	 	return;
	}
	sprintf(slist,"--samplelist %s", samplelist);
	sprintf(sample_id, "-s %d", info->current_sample_id);
	sprintf(port_id, "-p %d", info->current_port_id);
	sprintf(host,"-h %s", info->current_hostname);
	sprintf(output,"-o %s", filename);
	
	sprintf(cmd, "vjrender %s %s %s %s %s %s %s &", 
		render_to,
		port_id,
		host,
		sample_id,
		slist,
		output,
		editlist );
	display("Launched %s",cmd);
	
	system(cmd);

	
}


void on_log2_activate(GtkMenuItem *menuitem, gpointer user_data) {
	gtk_widget_show( Log_window );
}

void
on_show_sample_window1_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data) { 

	gtk_widget_show( Sample_window );
}

void
on_show_effects_1_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {
	gtk_widget_show( Effects_window);
	vj_comm_preload();
}

void
on_show_effects_chain1_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {
	gtk_widget_show( EffectChain_window );

}

void
on_show_effects_controls1_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {
	gtk_widget_show( EffectControls_window );
}


void
on_show_tag_window1_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {
	gtk_widget_show( TAG_window );
	tag_get_info();
}

void
on_TAG_v4l_device_combo_selection_get  (GtkWidget       *widget,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data) { }

void
on_TAG_v4l_device_entry_activate       (GtkEditable     *editable,
                                        gpointer         user_data) { }

void
on_TAG_uv4mpeg_applications_combo_selection_get
                                        (GtkWidget       *widget,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data) { }

void
on_TAG_yuv4mpeg_application_name_activate
                                        (GtkEditable     *editable,
                                        gpointer         user_data) { }

void
on_TAG_yuv4mpeg_filename_activate      (GtkEditable     *editable,
                                        gpointer         user_data) { }


void
on_TAG_yuv4mpeg_browse_clicked         (GtkButton       *button,
                                        gpointer         user_data) {



	gtk_widget_show( yuvselect_dialog);

}


gboolean
on_rgb24_select_delete_event(GtkWidget *w, GdkEvent *e, gpointer user_data) {
  gtk_widget_hide( rgb24select_dialog);
  return (TRUE);
}

gboolean
on_yuv4mpeg_select_delete_event        (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data) {
	gtk_widget_hide( yuvselect_dialog);
	return (TRUE);
}


void
on_cancel_button8_clicked              (GtkButton       *button,
                                        gpointer         user_data) {

	gtk_widget_hide( yuvselect_dialog);
} 

void on_cancel_button9_clicked (GtkButton *button, gpointer user_data) {
	gtk_widget_hide( rgb24select_dialog );
}

void
on_TAG_yuv4mpeg_mplayer_disable_audio_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data) { }

void
on_TAG_yuv4mpeg_mplayer_flip_toggled   (GtkToggleButton *togglebutton,
                                        gpointer         user_data) { }

void
on_TAG_yuv4mpeg_mplayer_auto_toggled   (GtkToggleButton *togglebutton,
                                        gpointer         user_data)  { }

void
on_TAG_yuv4mpeg_veejay_dummy_toggled   (GtkToggleButton *togglebutton,
                                        gpointer         user_data) { }

void
on_TAG_yuv4mpeg_veejay_disable_audio_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data) { }

void
on_TAG_yuv4mpeg_veejay_SDL_toggled     (GtkToggleButton *togglebutton,
                                        gpointer         user_data) { }

void
on_TAG_yuv4mpeg_veejay_DirectFB_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data) { }

void
on_TAG_vloopback_use_device_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();
	info->vloopback_tag->use_app = 0;
}

void
on_TAG_vloopback_application_toggled   (GtkToggleButton *togglebutton,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();
	info->vloopback_tag->use_app = 1;

}

void
on_TAG_vloopback_application_combo_selection_get
                                        (GtkWidget       *widget,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data) { }

void
on_TAG_vloopback_use_application_entry_activate
                                        (GtkEditable     *editable,
                                        gpointer         user_data) { }

void
on_TAG_vloopback_combo_use_device_selection_get
                                        (GtkWidget       *widget,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data) { }

void
on_TAG_vloopback_device_entry_activate (GtkEditable     *editable,
                                        gpointer         user_data) { }

void
on_TAG_vloopback_veejay_disable_audio_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data) { }

void
on_TAG_vloopback_veejay_SDL_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data) { }

void
on_TAG_vloopback_veejay_DirectFB_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data) { }

void
on_TAG_new_input_cancel_clicked        (GtkButton       *button,
                                        gpointer         user_data)  {

	gtk_widget_hide( GTK_WIDGET(TAG_window));
}

void
on_TAG_new_input_apply_clicked         (GtkButton       *button,
                                        gpointer         user_data) {

	GtkWidget *n = lookup_widget(TAG_window, "TAG_notebook");
	int page = gtk_notebook_get_current_page( GTK_NOTEBOOK(n));
	switch(page) {
		case 0: v4l_create(); break;
		case 1: yuv4mpeg_create(); break;
		case 2: vloopback_create(); break;
		case 3: rgb24_create(); break;
	}
	
	gtk_widget_hide( GTK_WIDGET(TAG_window));
}

gboolean
on_EffectControls_window_delete_event  (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data) {

	gtk_widget_hide( EffectControls_window );
	return (TRUE);
}

gboolean
on_Effects_window_delete_event         (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data) {
	gtk_widget_hide( Effects_window );
	return (TRUE);
}

gboolean
on_EffectChain_window_delete_event     (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data) {
	gtk_widget_hide( EffectChain_window );
	return (TRUE);
}

gboolean
on_Sample_window_delete_event          (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data) {
	gtk_widget_hide( Sample_window );
	return (TRUE);
}

gboolean
on_Tag_window_delete_event             (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data) {
	gtk_widget_hide( TAG_window );
	return (TRUE);
}

void
on_editlist_1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {
	gtk_widget_show( Editlist_window );
	get_editlist_info();
}

gboolean
on_EditList_window_delete_event        (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data) {
	gtk_widget_hide( Editlist_window );
	return (TRUE);
}


void
on_TAG_vloopback_application_input_selection_get
                                        (GtkWidget       *widget,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data) {


}

void
on_resize_video_screen1_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)  {
	unsigned int width, height;
	int res;
	char cmd[MESSAGE_SIZE];

	sprintf(cmd, "%d", MSG_REQ_VIDEO_PROPERTIES);
	vj_comm_write(cmd);

	vj_comm_read(cmd);
	sscanf(cmd, "%d %d %d", &res, &width , &height);

	if(res != MSG_REQ_VIDEO_PROPERTIES) {
	}
	else {
		vjcomm *info = vj_comm_get_info();
		char word[6];
		sprintf(word, "%d", width);
		gtk_label_set_text(GTK_LABEL(info->resize->org_width), word);
		sprintf(word, "%d", height);
		gtk_label_set_text(GTK_LABEL(info->resize->org_height), word);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(
			info->resize->new_width), (gint) width);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(
			info->resize->new_height), (gint) height);
		info->resize->ratio_val = ( (gdouble)width / (gdouble)height);
	}


	gtk_widget_show(Resize_window);

}

void
on_resize_cancel_clicked               (GtkButton       *button,
                                        gpointer         user_data) { 

	gtk_widget_hide(Resize_window);
}

void
on_resize_reset_clicked                (GtkButton       *button,
                                        gpointer         user_data) {
	unsigned int width, height;
	int res;
	char cmd[MESSAGE_SIZE];

	sprintf(cmd, "%d", MSG_REQ_VIDEO_PROPERTIES);
	vj_comm_write(cmd);

	vj_comm_read(cmd);
	sscanf(cmd, "%d %d %d", &res, &width , &height);

	if(res != MSG_REQ_VIDEO_PROPERTIES) {
	}
	else {
		vjcomm *info = vj_comm_get_info();
		char word[6];
		sprintf(word, "%d", width);
		gtk_label_set_text(GTK_LABEL(info->resize->org_width), word);
		sprintf(word, "%d", height);
		gtk_label_set_text(GTK_LABEL(info->resize->org_height), word);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(
			info->resize->new_width), (gint) width);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(
			info->resize->new_height), (gint) height);
		info->resize->ratio_val = ( (gdouble)width / (gdouble)height);
	}
	
}

void
on_resize_apply_clicked                (GtkButton       *button,
                                        gpointer         user_data) {

	char cmd[MESSAGE_SIZE];
	vjcomm *info = vj_comm_get_info();
	int w = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(info->resize->new_width));
	int h = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(info->resize->new_height));

	sprintf(cmd, "%d %d %d", MSG_CMD_RESIZE_SCREEN, w, h);
	vj_comm_write(cmd);


}


gboolean
on_resize_width_spin_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
	vjcomm *info = vj_comm_get_info();
	if(GTK_TOGGLE_BUTTON(info->resize->ratio)->active) {
		//gint height = gtk_spin_button_get_value_as_int(
		//	GTK_SPIN_BUTTON(info->resize->new_height));
		gint width = gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(info->resize->new_width));
		gint new_height = (gint)( (gdouble) width / info->resize->ratio_val + 0.5);
		gtk_spin_button_set_value(
			GTK_SPIN_BUTTON(info->resize->new_height), new_height);
	}

	return (TRUE);
}

gboolean
on_resize_height_spin_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data){
	vjcomm *info = vj_comm_get_info();
	if(GTK_TOGGLE_BUTTON(info->resize->ratio)->active) {
		gint height = gtk_spin_button_get_value_as_int(
			GTK_SPIN_BUTTON(info->resize->new_height));
		//gint width = gtk_spin_button_get_value_as_int(
		//	GTK_SPIN_BUTTON(info->resize->new_width));
		gint new_width = (gint)(info->resize->ratio_val * height);
		gtk_spin_button_set_value(
			GTK_SPIN_BUTTON(info->resize->new_width), new_width);
	}

	return (TRUE);
}

void
on_resize_ratio_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data){ }




void
on_TAG_recmaxframes_changed            (GtkEditable     *editable,
                                        gpointer         user_data) {
		
}

gboolean
on_TAG_recmaxframes_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
   return FALSE;
}

void
on_TAG_recautoactivate_toggled         (GtkToggleButton *togglebutton,
                                        gpointer         user_data) {
}

void
on_TAG_recstartstop_toggled            (GtkToggleButton *togglebutton,
                                        gpointer         user_data) {
    char cmd[MESSAGE_SIZE];
    vjcomm *info = vj_comm_get_info();
    GtkWidget *w = lookup_widget(Sample_window, "TAG_recmaxframes");
    GtkWidget *a = lookup_widget(Sample_window, "TAG_recautoactivate");
    int max_frames = gtk_spin_button_get_value_as_int(
		GTK_SPIN_BUTTON(w));
    int auto_switch = 0;
    
    if(info->playback_mode != VJ_PLAYBACK_MODE_TAG) {
	display("Switch to a Tag to start recording it or use Offline Tag record");
        return;
	} 
   
    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(a))) {
	auto_switch = 1;
    }
	
    if(gtk_toggle_button_get_active(togglebutton)) {
	sprintf(cmd, "%d %d %d", MSG_CMD_TAG_ONLINE_REC_START,	
		max_frames, auto_switch);
	vj_comm_write(cmd);	
    }
    else {
	sprintf(cmd, "%d", MSG_CMD_TAG_ONLINE_REC_STOP);	
	vj_comm_write(cmd);
    }
}

void
on_TAG_offlneautoactivate_toggled      (GtkToggleButton *togglebutton,
                                        gpointer         user_data) {

}

void
on_TAG_offlinestartstoprecord_toggled  (GtkToggleButton *togglebutton,
                                        gpointer         user_data) {

    char cmd[MESSAGE_SIZE];
    vjcomm *info = vj_comm_get_info();
    GtkWidget *w = lookup_widget(Sample_window, "TAG_offlinemaxframesspin");
    GtkWidget *a = lookup_widget(Sample_window, "TAG_offlineautoactivate");
    int max_frames = gtk_spin_button_get_value_as_int(
		GTK_SPIN_BUTTON(w));
    int auto_switch = 0;
    
    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(a))) {
	auto_switch = 1;
    }
	
    if(gtk_toggle_button_get_active(togglebutton)) {
	sprintf(cmd, "%d %d %d %d", MSG_CMD_TAG_OFFLINE_REC_START,	
		info->selected_sample,
		max_frames, auto_switch);
	vj_comm_write(cmd);	
    }
    else {
	sprintf(cmd, "%d %d", MSG_CMD_TAG_OFFLINE_REC_STOP, info->selected_sample);	
	vj_comm_write(cmd);
    }

}

void
on_SE_edit_record_frames_changed       (GtkEditable     *editable,
                                        gpointer         user_data) {


}

gboolean
on_SE_edit_record_frames_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
 return FALSE;

}

void
on_SE_record_auto_switch_toggled       (GtkToggleButton *togglebutton,
                                        gpointer         user_data) {

}

void
on_SE_startstop_recording_toggled      (GtkToggleButton *togglebutton,
                                        gpointer         user_data) {
    char cmd[MESSAGE_SIZE];
    vjcomm *info = vj_comm_get_info();
    GtkWidget *w = lookup_widget(Sample_window, "SE_edit_record_frames");
    GtkWidget *a = lookup_widget(Sample_window, "SE_record_auto_switch");
    int max_frames = gtk_spin_button_get_value_as_int(
		GTK_SPIN_BUTTON(w));
    int auto_switch = 0;
    if(info->playback_mode != VJ_PLAYBACK_MODE_SAMPLE) {
	display("Switch to a Sample to start recording it.");
        return;
	}
    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(a))) {
	auto_switch = 1;
    }
	
    if(gtk_toggle_button_get_active(togglebutton)) {
	sprintf(cmd, "%d %d %d", MSG_CMD_SAMPLE_REC_START,	
		max_frames, auto_switch);
	vj_comm_write(cmd);	
    }
    else {
	sprintf(cmd, "%d", MSG_CMD_SAMPLE_REC_STOP);	
	vj_comm_write(cmd);
    }
 
}

gboolean
on_EffectChain_fadespin_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data) {
  return FALSE;
}

void
on_EffectChain_fadespin_changed        (GtkEditable     *editable,
                                        gpointer         user_data){

}

void
on_EffectChain_gofade_clicked          (GtkButton       *button,
                                        gpointer         user_data) {

  GtkWidget *w = lookup_widget(EffectChain_window, "EffectChain_fadespin");
  gint val = gtk_spin_button_get_value_as_int(
		GTK_SPIN_BUTTON(w));
  char cmd[MESSAGE_SIZE];
  sprintf(cmd, "%d %d %d", MSG_CMD_CHAIN_FADE, val,0);
  vj_comm_write(cmd);
}


void
on_EffectChain_FadeOut_clicked         (GtkButton       *button,
                                        gpointer         user_data) {
  GtkWidget *w = lookup_widget(EffectChain_window, "EffectChain_fadespin");
  gint val = gtk_spin_button_get_value_as_int(
		GTK_SPIN_BUTTON(w));
  char cmd[MESSAGE_SIZE];
  sprintf(cmd, "%d %d %d", MSG_CMD_CHAIN_FADE, val,1);
  vj_comm_write(cmd);


}

void
on_TAG_rgb24_filename_activate         (GtkEditable     *editable,
                                        gpointer         user_data) {

}

void
on_TAG_rgb24_browse_clicked            (GtkButton       *button,
                                        gpointer         user_data) {

   
	gtk_widget_show( rgb24select_dialog);

}
gboolean
on_SampleList_window_delete_event      (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data) {
	gtk_widget_hide( SampleList_window );
	return (TRUE);
}


void
on_sample_tag_list1_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data) {
  	gtk_widget_show( SampleList_window );
}
