/* (C) Niels Elburg <elburg@hio.hen.nl>
 * 
 * This program is free software you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */


#ifndef VJ_COMM_H
#define VJ_COMM_H
#include <gtk/gtk.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include "tracker.h"
#include "vj-global.h"


typedef struct {
	GtkWidget *list;
	GtkWidget *spin;
	gdouble ratio_val;

} vj_render;

typedef struct {
	char *applications;
	char *device;
	char *parameters;
	GtkWidget *device_entry;
	GtkWidget *device_combo;
	GtkWidget *application_entry;
	GtkWidget *application_combo;
	GtkWidget *application_input;
	GtkWidget *application_device;
	int use_app;
} vj_vloopback_tag;

typedef struct {
	GtkWidget *new_width;
	GtkWidget *new_height;
	GtkWidget *org_width;
	GtkWidget *org_height;
	GtkWidget *ratio;
	gdouble ratio_val;
} vj_resize;

typedef struct {
	char *device;
	GtkWidget *device_entry;
	GtkWidget *device_combo;
	
} vj_v4l_tag;

typedef struct {
	char *applications;
	char *filename;
	char *parameters;
	GtkWidget *application_combo;
	GtkWidget *application_entry;
	GtkWidget *filename_entry;
} vj_yuv4mpeg_tag;




typedef struct vj_raw_edit_t {
	GtkWidget *total_frames_label;
	GtkWidget *total_duration_label;
	GtkWidget *start_frame_spin;
	GtkWidget *end_frame_spin;
	GtkWidget *paste_at_spin;
	GtkWidget *sel_start_button;
	GtkWidget *sel_end_button;
	GtkWidget *paste_at_button;
	GtkWidget *copy_button;
	GtkWidget *cut_button;
	GtkWidget *crop_button;
	GtkWidget *del_button;
	glong sel_paste_at;
	glong sel_end_at;
	glong sel_start_at;
	gint	sel_start_pressed;
	gint 	sel_end_pressed;
	gint	sel_paste_pressed;	
} vj_raw_edit;

typedef struct {
	struct hostent *he;
	struct sockaddr_in server_addr;
	int handle;
} vj_connection;


/*
typedef struct vj_sample_list_t {
	
} vj_sample_list;

typedef struct vj_pattern_editor_t {

} vj_pattern_editor;
*/
typedef struct vj_pattern_instrument_t {
	gint selected_row;
	gint selected_column;
	GList *LumaTransitions;
	GList *TranslationTransitions;
	GList *MotionTransitions;
	GList *ImageEffects;
	GList *VideoEffects;
	GList *StaticEffects;
} vj_pattern_instrument;

typedef struct vj_navigator_t {
	GtkWidget *current_frame_label;
	GtkWidget *current_timecode_label;
	GtkWidget *blend_combobox;
	GtkWidget *vloopback_toggle;
	GtkWidget *audio_toggle;	
	GtkWidget *effect_toggle;
	GtkWidget *speedlock_toggle;
	GtkWidget *enable_v4ltoggle;
} vj_navigator;

typedef struct vj_pattern_editor_t {
	gint row;
	gint column;
} vj_pattern_editor;

typedef struct vj_sample_edit_t {
	GtkWidget *current_chain_label;
	GtkWidget *sample_label;
	GtkWidget *numframes_label;
	GtkWidget *duration_label;
	GtkWidget *start_frame_spin;
	GtkWidget *end_frame_spin;
	GtkWidget *playback_speed_spin;
	GtkWidget *next_sample_spin;
	GtkWidget *max_loops_spin;
	GtkWidget *loop_button, *noloop_button, *pingpong_button;
	GtkWidget *sample_controls;
	//GtkWidget *combo_box_image_effects;
	//GtkWidget *combo_box_video_effects;
	GtkWidget *image_effects;
	GtkWidget *video_effects;
	GtkWidget *effect_chain;
	/* Effect Chain Control buttons and sliders */
	GtkWidget *slider_p0, *slider_p1, *slider_p2, *slider_p3, *slider_p4;
	GtkWidget *slider_p5, *slider_p6, *slider_p7, *slider_p8;
	GtkWidget *spinbox_p0, *spinbox_p1, *spinbox_p2, *spinbox_p3, *spinbox_p4;
	GtkWidget *spinbox_p5, *spinbox_p6, *spinbox_p7, *spinbox_p8;
	GtkWidget *bypass_button;
	GtkWidget *render_button ,*volume_slider, *volume_spin;
	GtkWidget *tag_active;
	GtkWidget *tag_source;
	GtkWidget *tag_sourcename;
	GtkWidget *tag_id;
	GtkWidget *source;
	GtkWidget *channel;
	GtkWidget *sample_type;
	gint slider_down;
	gint tag_type;
	gint speed_pressed;
	gint next_sample_pressed;
	gint max_loops_pressed;
	gint start_frame_pressed;
	gint end_frame_pressed;
	gint selected_effect_id;
	gint new_image_effect;
	gint new_video_effect;
	gint activate_tag;
	gint source_number;
	gint channel_number;
	gint bypass_update;
	gint bypass_entry;
} vj_sample_edit;


typedef struct vjcomm_t {
	vj_navigator *Navigator;
	vj_raw_edit *RawEdit;
	vj_sample_edit *SampleEdit;
	vj_pattern_editor *PatternEdit;
	vj_pattern_instrument *PatternInstrument;
	vj_yuv4mpeg_tag *yuv4mpeg_tag;
	vj_vloopback_tag *vloopback_tag;
	vj_v4l_tag *v4l_tag;
	vj_render *render;
	vj_resize *resize;
	GtkWidget *video_bar;
	GtkWidget *status_text;
	GtkWidget *PatternEditor;
	GtkWidget *SampleList;
	GtkWidget *EditList;
	
	//GList *EditList;
	GtkWidget *playmode_box;
	GList *video_devices;
	GList *effects;
	GList *image_effects;	/* contains image effects descriptions for combobox*/
	GList *video_effects;
	
	Tracker *tracker;
	glong frame_num;
	glong total_frames;
	gint total_samples;
	gdouble fps;
	gint slider_pause;
	gint hscale_down;
	glong selection_start;
	glong selection_end;
	gint current_sample_id;
	gint selected_sample;
	gint selected_row;
	glong sample_start;
	glong sample_end;
	gint playback_speed;
	gint chain_entry;
	gint next_sample;
	gint max_loops;
	glong sel_paste_at;
	glong sel_start_at;
	glong sel_end_at;
	gint looptype;
	glong new_sample_start;
	glong new_sample_end;
	gint pattern_size;
	gint default_blend_id;
	gint pattern_preview_mode;
	gint current_row;
	gint current_col;
	gint effect_chain;
	gchar cmd_msg[MESSAGE_SIZE];
	gchar req_msg[MESSAGE_SIZE];
	gchar ren_msg[MESSAGE_SIZE];
	gchar stat_msg[MESSAGE_SIZE];
	gint entry_active;
	gint playback_mode;
	gint current_effect_id;
	gint total_tags;
	gint selected_sample_type;
	gint tag_is_active;
	vj_connection *vjc;
	vj_connection *status;
	char pattern_row_str[1024];
	int effects_loaded;
	gint active_render_entry;
	gint current_port_id;
	char current_hostname[1024];
	gint top_effects_toggle;
	GtkWidget *logmsg;
	char veejay_path[1024];
	gint log_on;
} vjcomm;
/* update_sensitive_effect_controls(params, extra_frame) */

GtkWidget *MainWindow;
GtkWidget *SettingsWindow;
GtkWidget *TagWindow;
GtkWidget *EffectControls_window;
GtkWidget *Effects_window;
GtkWidget *EffectChain_window;
GtkWidget *Sample_window;
GtkWidget *SampleList_window;
GtkWidget *TAG_window;
GtkWidget *Editlist_window;
GtkWidget *yuvselect_dialog;
GtkWidget *rgb24select_dialog;
GtkWidget *Resize_window;
GtkWidget *Log_window;

vjcomm *vj_comm_get_info();
void vj_comm_max_effects();
int vj_comm_connect(char *, int);
int vj_comm_status_connect(char *, int);
void vj_comm_preload();
int valid_mode_sample();
int valid_mode_pattern();
int valid_mode_tag();
void vj_comm_init();
void get_editlist_info(); 
void msg_no_sample(); 
void enable_raw_edit();
void disable_raw_edit();
void update_active_entry(int num);
void update_pattern_play(int num);
void update_sample_play(int num);
void update_effect_control_values(); 
void update_sensitive_effect_controls();
void update_chain_entry(gint n);
void update_total_frames(glong n); 
void update_effect_parameters(); 
void update_current_frame(glong n);
void update_sample_speed(gint n);
void update_current_sample(gint n);
void update_sample_start(glong n);
void update_sample_end(glong n); 
void update_max_loops(gint nr);
void update_effect_control_limits(GtkWidget *widget, int param) ;
void update_pattern_row(gint n);
void get_all_samples(int numsamples); 
void update_effect_chain_contents(); 
void update_pattern_size(gint n);
void update_next_sample(gint nr);
void update_sample_edit(glong new_start_frame, glong new_end_frame); 
void update_effect_chain(gint n); 
void process_status_information(gpointer data, gint fd, GdkInputCondition condition);
void update_vscrollbar(Tracker *t, int patpos, int patlen, int disprows);
void update_hscrollbar(Tracker *t, int leftchan, int numchans, int dispchans);
void update_mainmenu_blockmark( Tracker *t, int state );
void on_vscroll_changed( GtkAdjustment *adjustment, gpointer user_data);
void on_hscroll_changed( GtkAdjustment *adjustment, gpointer user_data);
void on_videobar_changed(GtkAdjustment *adjustment, gpointer user_data);
void update_effect_control(GtkWidget *widget, gint param_nr);
void on_slider_changed(GtkAdjustment *adjustment, gint param_nr);
void on_slider_p0_changed( GtkAdjustment *adjustment, gpointer user_data);
void on_slider_p1_changed( GtkAdjustment *adjustment, gpointer user_data);
void on_slider_p2_changed( GtkAdjustment *adjustment, gpointer user_data);
void on_slider_p3_changed( GtkAdjustment *adjustment, gpointer user_data);
void on_slider_p4_changed( GtkAdjustment *adjustment, gpointer user_data);
void on_slider_p5_changed( GtkAdjustment *adjustment, gpointer user_data);
void on_slider_p6_changed( GtkAdjustment *adjustment, gpointer user_data);
void on_slider_p7_changed( GtkAdjustment *adjustment, gpointer user_data);
void on_slider_p8_changed( GtkAdjustment *adjustment, gpointer user_data);
void sample_description_get(); 
void samplelist_update(); 
void update_slider_control(GtkWidget *widget, int param_nr); 
void update_v4l_sampleedit_down();
void update_v4l_sampleedit_up();
void update_current_entry_contents( gint *effect_arg) ;
int vj_comm_read(char *buf);
int vj_comm_write(char *buf);
int vj_comm_set_conn(char *name, int port); 
void get_all_tags(int nr);
void update_effect_control_values();
void vj_comm_exit();
void v4l_create();
void yuv4mpeg_create();
void vloopback_create();
void vj_comm_log_on();
void tag_get_info();
#endif 
