/* global definition file (C) Niels Elburg <elburg@hio.hen.nl>
 * 
 * This program is free software you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#ifndef VJ_GLOBAL_H
#define VJ_GLOBAL_H

/*
 
   This header file defines the actions for the GUI.
   It looks like a transaction system on a 1 on 1 basis (gui <-> veejay)
   The named pipes are defined in libveejayvj.c 
 
   status:
   	current_frame, total_frames, samples, sample_id,
	speed, looptype, chain_on, speedlock
	
 
   request: (REQ)
	responds with a message (data)
 
 
   command: (CMD)
	do action, no response	
 
*/

enum {
    VJ_PLAYBACK_MODE_PLAIN = 1,
    VJ_PLAYBACK_MODE_SAMPLE = 2,
    VJ_PLAYBACK_MODE_PATTERN = 3,
    VJ_PLAYBACK_MODE_TAG = 4,
    VJ_PLAYBACK_MODE_MEM = 5,
};

enum {
    VJ_MAX_V4L_DEVICES = 2,
    VJ_MAX_IN_STREAMS = 4,
    VJ_MAX_OUT_STREAMS = 1,
    VJ_MAX_VLOOPBACK_PIPES = 8,
};


/* request messages, global */

#define MSG_REQ_SET_SAMPLEEND			3003	/* sets end frame of sample */
#define MSG_REQ_SET_CHAINENTRY			3004	/* sets current chain entry of sample */
#define MSG_REQ_SAMPLE_GET_RENDER_LIST		3051
#define MSG_REQ_TOGGLE_PLAYLIST			3053
#define MSG_REQ_ADD_EFFECT			3007	/* add effect to chain of sample */
#define MSG_REQ_GET_CHAINENTRY			3008	/* reads effect parameters of chain entry , sample */
#define MSG_REQ_GET_RENDERING_ENTRY		3360
#define MSG_REQ_DUP_CHAIN_ENTRY			3410
#define MSG_REQ_SET_SUBCHAIN			3414
#define MSG_REQ_GET_SAMPLES			3101	/* gets a number of samples */
#define MSG_REQ_NEW_VLOOPBACK_BY_PIPE		3100    /* make a new tag */
#define MSG_REQ_NEW_VLOOPBACK_BY_NAME           3102
#define MSG_REQ_NEW_YUV4MPEG			3103
#define MSG_REQ_NEW_V4L				3104
#define MSG_REQ_NEW_RAW				3105
#define MSG_REQ_GET_TAGS			3200
#define MSG_REQ_GET_NR_MODS			3300	/* number of modules available */
#define MSG_REQ_GET_MOD				3301	/* description of a modules identified by number */
#define MSG_REQ_GET_NFILES			3302
#define MSG_REQ_FLOOD_EDITLIST			3303
#define MSG_REQ_FLOOD_TAGS			3305
#define MSG_REQ_FLOOD_SAMPLES			3304    /* send all sample list items */
#define MSG_REQ_GET_DEVICES			3400    /* query devices */
#define MSG_REQ_VIDEO_PROPERTIES		3500    /* get video properties */
#define MSG_REQ_GET_ALL_EFFECTS			3600
#define MSG_REQ_EFFECT_COUNT			3601
#define MSG_REQ_GET_EFFECTLIMITS		3603
#define MSG_REQ_GET_EFFECTDEFAULTS		3604
#define MSG_REQ_GET_SAMPLEDESCR			3709
#define MSG_REQ_GET_SAMPLE_INFO			3711
#define MSG_REQ_ADD_VIDEO_FILE			3807
#define MSG_REQ_ADD_VIDEO_SAMPLE		3816
#define MSG_REQ_SAVE_EDITLIST			3908
#define MSG_REQ_DEL_VIDEO_FILE			3809
#define MSG_REQ_GET_EFFECT			3605	/* effect description */
#define MSG_REQ_GET_EFFECT_LIMIT		3606	/* get effect's limits, FIXME */
#define MSG_REQ_COPY_SAMPLE			3005	/* copies a sample */
#define MSG_REQ_GET_NR_SAMPLES			3006	/* get number of samples */
#define MSG_REQ_GET_NR_TAGS			3201
#define MSG_REQ_GET_EFFECT_INFO			3615	/* effect description */
#define MSG_REQ_OPEN				3650	/* open samplelist */
#define MSG_REQ_SAVE				3660	/* save samplelist */
#define MSG_REQ_GET_FULL_CHAIN			3610	/* return effect parameters */
#define MSG_REQ_GET_FULL_NCHAIN			3611
#define MSG_REQ_EDIT_VIDEO			3900	/* cut,copy,paste,crop, del */


//#define MSG_CMD_GET_BLEND_TYPES		8215
//#define MSG_CMD_EDITLIST_ADD_SAMPLE		8218
#define MSG_SAMPLE_REQ_EFFECT_PARAM		3607

/* pattern */
#define MSG_CMD_PATTERN_GET_ROW			4200
#define MSG_CMD_PATTERN_SET_ROW			4201
#define MSG_PATTERN_GET_EFFECT_PARAM		4508
#define MSG_PATTERN_ADD_SAMPLE			4509
#define MSG_PATTERN_SET_EFFECT_PARAM		4505


/* global */
#define MSG_CMD_SET_PLAYMODE			1001
#define MSG_CMD_TOGGLE_OUTSTREAM		1002
#define MSG_CMD_STREAM_IN_START			1003
#define MSG_CMD_STREAM_OUT_START		1004
#define MSG_CMD_STREAM_IN_STOP			1005
#define MSG_CMD_STREAM_OUT_STOP			1006
#define MSG_CMD_STREAM_IN_CREATE		1007	//filename
#define MSG_CMD_STREAM_OUT_CREATE		1008
#define MSG_CMD_RESIZE_SCREEN			1009
#define MSG_CMD_WINDOW_CREATE			1010
#define MSG_CMD_WINDOW_TOGGLE			1011
#define MSG_CMD_WINDOW_DELETE			1012
#define MSG_CMD_QUIT				1013	/* leave veejay */
#define MSG_CMD_GOODBYE				1014    /* close connection with veejay */
#define MSG_CMD_MOD_MAKE			1015	/* generates a number of samples (vjgen) */
#define MSG_CMD_MOD_SAMRAND			1016	/* sample sequencer */
#define MSG_CMD_SET_TAG				1017
#define MSG_CMD_PATTERN_PLAY			1018    /* not implemented */
#define MSG_CMD_DEL_ALL				1019	/* delete all samples */
#define MSG_CMD_DEL_SAMPLE			1020	/* deletes a sample */
#define MSG_CMD_GETWD				1021
#define MSG_CMD_PLAY_FORWARD 			1022	/* play forward video/sample/pattern */
#define MSG_CMD_PLAY_BACKWARD			1023	/* play backward video/sample/pattern */
#define MSG_CMD_FAST_FORWARD			1024	/* fast forward video/sample/pattern */
#define MSG_CMD_FAST_BACKWARD			1025	/* fast backward video/sample/pattern */
#define MSG_CMD_SKIPTOSTART			1026	/* go to beginning of video/sample/pattern */
#define MSG_CMD_SKIPTOEND			1027	/* go to end of video/sample/pattern */
#define MSG_CMD_PLAYPAUSE			1028	/* play / pause video/sample/pattern */
#define MSG_CMD_SET_FRAME			1029	/* set position video/pattern */
#define MSG_CMD_SET_SPEED			1030	/* set speed video/pattern */
#define MSG_CMD_RENDER_SAMPLE_TO	        1031    /* launch vjrender */

#define MSG_CMD_TOGGLE_SAMPLEPLAY		1032	/* toggle bypass samplelist */
 

/* sample related */
#define MSG_CMD_SET_SAMPLE_SOURCE		2001
#define MSG_CMD_SET_SAMPLE_CHANNEL		2002
#define MSG_CMD_SET_SAMPLEDESCR			2003
#define MSG_CMD_SAMPLE_SET_VOLUME		2004
#define MSG_CMD_SET_RENDERING_ENTRY		2020
#define MSG_CMD_SAMPLE_SET_RENDERING_ENTRY	2005
#define MSG_CMD_SAMPLE_SET_FRAMEDUP		2006
#define MSG_CMD_SET_START_FRAME			2007	/* update start frame of sample */
#define MSG_CMD_SET_END_FRAME			2008	/* update end frame of sample */
#define MSG_CMD_SET_SAMPLE_LOOPS		2009	/* set number of loops , sample */
#define MSG_CMD_SET_SAMPLE_RENDER_ENTRY	        2010
#define MSG_CMD_SAMPLE_RENDER_DEL		2011
#define MSG_CMD_SAMPLE_RENDER_MOVE		2012
#define MSG_CMD_SET_LOOPTYPE			2013	/* toggle ping ping, sample */
#define MSG_SAMPLE_SET_EFFECT_PARAM		2014
#define MSG_CMD_TRIM_SAMPLE			2015
#define MSG_CMD_TOGGLE_AUDIO			2016	/* toggle audio video/sample/pattern */
#define MSG_CMD_SET_SAMPLE			2017	/* activate a sample */
#define MSG_CMD_SET_SAMPLESTART			2018	/* sets start frame of sample */
#define MSG_CMD_SET_SAMPLE_NEXT			2019	/* set next sample , sample */



/* chain related */
#define MSG_CMD_SET_CHAIN_AUDIO			2101
#define MSG_CMD_SET_CHAIN_AUDIO_VOLUME		2102
#define MSG_CMD_RESET_CHAIN_ENTRY		2103
#define MSG_CMD_SET_ENTRY_DEFAULTS		2104
#define MSG_CMD_BYPASS_CHAIN_ENTRY		2105
#define MSG_CMD_ADD_SET_EFFECT			2106
#define MSG_CMD_CLEAR_ENTRY			2107
#define MSG_CMD_SET_CHAINARG			2108	/* sets arguments for current chain entry of sample */
#define MSG_CMD_DEL_CHAINENTRY			2109	/* resets a chain entry */
#define MSG_CMD_TOGGLE_CHAIN			2110	/* toggle effect chain, sample */

/* output driver RAW stream */

#define MSG_CMD_SET_RENDER_NONSTOP		9001     /* only for -O5 */
#define MSG_CMD_SET_RENDER_INTERACTIVE		9002    /* idem */
#define MSG_CMD_RENDER_FRAME			9003	/* idem */


/* tag */
#define MSG_CMD_SAMPLE_REC_START	        2040
#define MSG_CMD_SAMPLE_REC_STOP	                2050
#define MSG_CMD_CHAIN_FADE			9009
#define MSG_CMD_TAG_ONLINE_REC_START		2160
#define MSG_CMD_TAG_ONLINE_REC_STOP		2161
#define MSG_CMD_TAG_OFFLINE_REC_START		2170
#define MSG_CMD_TAG_OFFLINE_REC_STOP		2171

#define MSG_CMD_SET_TAG_ACTIVE			2111


#define MSG_CMD_INC_P0 4001
#define MSG_CMD_DEC_p0 4002
#define MSG_CMD_INC_p1 4003
#define MSG_CMD_DEC_p1 4004
#define MSG_CMD_INC_p2 4005
#define MSG_CMD_DEC_p2 4006
#define MSG_CMD_INC_p3 4007
#define MSG_CMD_DEC_p3 4008
#define MSG_CMD_INC_p4 4009
#define MSG_CMD_DEC_p4 4010


/* other */

#define SAMPLE_MAX_EFFECTS	10
#define PATTERN_MAX_TRACKS	3
#define CELL_LENGTH		30
#define ROW_SIZE		(PATTERN_MAX_TRACKS * CELL_LENGTH)
#define PATTERN_DEFAULT_LENGTH  (25*60)	/* have aprox. 1 minute */
#define EDIT_CUT		101
#define EDIT_PASTE_AT		102
#define EDIT_COPY		103
#define EDIT_CROP		104
#define EDIT_DEL		105
#define MAX_EFFECTS 		79
#define MESSAGE_SIZE		512	/* enough for my needs */



#endif
